@@ -10,10 +10,17 @@ sub decode {
 
     # RST_STREAM associated with stream
     if ( $frame_ref->{stream} == 0 ) {
+        tracer->error("Received reset stream with stream id 0");
         $con->error(PROTOCOL_ERROR);
         return undef;
     }
 
+    if ( $length != 4 ) {
+        tracer->error("Received reset stream with invalid length $length");
+        $con->error(FRAME_SIZE_ERROR);
+        return undef;
+    }
+
     my $code = unpack( 'N', substr( $$buf_ref, $buf_offset, 4 ) );
 
     tracer->debug( "Receive reset stream with error code "
